/*
 * Copyright (c) 2000, 2001 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Clack.
 * 
 * Clack is free software, also known as "open source."  You can redistribute
 * it and/or modify it under the terms of the GNU General Public License
 * (GPL), version 2, as published by the Free Software Foundation (FSF).
 * 
 * Clack is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GPL for more details.  You should have
 * received a copy of the GPL along with Clack; see the file COPYING.  If
 * not, write to the FSF, 59 Temple Place #330, Boston, MA 02111-1307, USA.
 * 
 * Clack is modelled on and includes some code fragments from MIT's Click
 * system (http://www.pdos.lcs.mit.edu/click/) which is copyright (c)
 * 1999-2000 Massachusetts Institute of Technology and also GPL'ed.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

#include <clack/c/packet.h>
#include <strings.h>
#include <assert.h>
#ifdef OSKIT
#include <oskit/dev/dev.h>
#else
#define OSENV_PHYS_WIRED 0
#endif

static void clack_packet_free(clack_packet_t *p);

// Must include PHYS_WIRED if this packet is going to a device driver
// Use of AUTO_SIZE will prevent caching at the osenv_mem level
// #define CLACK_MEM_FLAGS (OSENV_PHYS_WIRED | OSENV_AUTO_SIZE)

#define CLACK_MEM_FLAGS OSENV_PHYS_WIRED

clack_packet_t *clack_packet_alloc(void)
{
        clack_packet_t *p = (clack_packet_t *)osenv_mem_alloc
                (sizeof(*p), CLACK_MEM_FLAGS, 0);
        return p;
}

static void clack_packet_free(clack_packet_t *p)
{
        assert(p->count == 0);
#ifdef  OSKIT
        if (p->bufio) {
                oskit_bufio_release(p->bufio);
        } else
#endif
                osenv_mem_free(p->data, CLACK_MEM_FLAGS, p->max);

        osenv_mem_free(p, CLACK_MEM_FLAGS, sizeof(*p));
}

void clack_packet_addref(clack_packet_t *p)
{
        assert(p->count);
        ++p->count;
}

void clack_packet_release(clack_packet_t *p)
{
        assert(p->count);
        if (--p->count == 0) {
                clack_packet_free(p);
        }
}

#ifdef OSKIT
clack_packet_t *bufio_to_clack_packet(oskit_bufio_t *b, clack_size_t size)
{
        clack_packet_t *p = clack_packet_alloc();
        if (p == NULL) return 0;

        oskit_bufio_addref(b);

        p->count = 1;
#ifdef  MEASURE
	p->measure = 0;
#endif
        p->bufio = b;
        p->start = 0;
        p->end   = p->max = size;
        oskit_bufio_map(b, &(p->data), 0, p->end); // ToDo: check return code

        // don't initialize paint
        bzero(&(p->anno), sizeof p->anno);
        // don't initialize ip_dst

        return p;
}
#endif

void* clack_packet_alloc_padded(clack_size_t size, clack_size_t padding)
{
        clack_size_t real_size;
        clack_packet_t *p = clack_packet_alloc();
        if (p == NULL) return 0;

        padding += 64;  // add some slop in
        real_size = padding + size + 64;  // and more slop

        p->data  = osenv_mem_alloc(real_size, CLACK_MEM_FLAGS, 0);
        // ToDo: check for null

#ifdef  OSKIT
        p->bufio = 0;
#endif
#ifdef  MEASURE
	p->measure = 0;
#endif
        p->count = 1;
        p->start = padding;
        p->end   = padding + size;
        p->max   = real_size;

        // don't initialize paint
        bzero(&(p->anno), sizeof p->anno);
        // don't initialize ip_dst

        return p;
}

void clack_packet_shift(clack_packet_t *p, clack_size_t size)
{
        assert(p->count);
        p->start += size;
}

void* clack_packet_unshift(clack_packet_t *p, clack_size_t size)
{
        assert(p->count);
        p->start -= size;
        return clack_packet_data(p);
}

