/*
 * Copyright (c) 2000, 2001 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Clack.
 * 
 * Clack is free software, also known as "open source."  You can redistribute
 * it and/or modify it under the terms of the GNU General Public License
 * (GPL), version 2, as published by the Free Software Foundation (FSF).
 * 
 * Clack is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GPL for more details.  You should have
 * received a copy of the GPL along with Clack; see the file COPYING.  If
 * not, write to the FSF, 59 Temple Place #330, Boston, MA 02111-1307, USA.
 * 
 * Clack is modelled on and includes some code fragments from MIT's Click
 * system (http://www.pdos.lcs.mit.edu/click/) which is copyright (c)
 * 1999-2000 Massachusetts Institute of Technology and also GPL'ed.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

#ifndef _CLACK_PACKET_H_
#define _CLACK_PACKET_H_

#include <clack/c/clack_types.h>
#include <netinet/in.h>			/* in_addr */
#ifdef OSKIT
#include <oskit/io/bufio.h>
#else
#include <netinet/in_systm.h>		/* misc types */
#endif


struct clack_anno {
        struct in_addr next_hop; // next hop in route (ie gateway)
        char           paint;
        int mac_broadcast : 1;
        int fix_ip_src    : 1;   // message to be sent back to previous hop
                                 // with (one of) this host's IP address on it
                                 // can't I use an address of 0.0.0.0 instead?
};

typedef struct clack_anno clack_anno_t;

struct clack_packet {
        clack_u32_t    count; // reference count
#ifdef  OSKIT
        oskit_bufio_t *bufio; // may be null
#endif
        void          *data;  // not null!

        /* 
         * Offsets in buffer: 0 <= start <= end <= max == b->size()
         * Partitions buffer into 3 regions:
         *   unused = [0..start)
         *   data   = [start..end)
         *   unused = [end..max)
         */
        clack_u32_t    start;
        clack_u32_t    end;
        clack_u32_t    max;

        struct clack_anno anno;
#ifdef  MEASURE
	char			measure;
	unsigned long long	created;
#ifdef	PERFMON
	unsigned long long	counter0;
	unsigned long long	counter1;
#endif
#endif
};

typedef struct clack_packet clack_packet_t;

#if 0
/* 
 * If clack_packet_{data,length} are included in the SigPacket bundletype,
 * the definitions go in packet.c.  Otherwise, they go here.
 */
#else
static inline
void* clack_packet_data(clack_packet_t *p)
{
        return (p->data + p->start);
}

static inline
clack_size_t clack_packet_length(clack_packet_t *p)
{
        return (p-> end - p->start);
}
#endif

clack_packet_t *clack_packet_alloc(void);
void clack_packet_addref(clack_packet_t*);
void clack_packet_release(clack_packet_t*);

#ifdef OSKIT
clack_packet_t *bufio_to_clack_packet(oskit_bufio_t *b, clack_size_t size);
#endif

// Allocate a packet with extra space in the header
// In fact, we add a bit more space yet!
void* clack_packet_alloc_padded(clack_size_t size, clack_size_t padding);

void clack_packet_shift(clack_packet_t *p, clack_size_t size);
void* clack_packet_unshift(clack_packet_t *p, clack_size_t size);
#define clack_packet_UNSHIFT(p,ty) ((ty*) clack_packet_unshift((p),sizeof (ty)))


#endif /* _CLACK_PACKET_H_ */
