/*
 * Copyright (c) 2000, 2001 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Clack.
 * 
 * Clack is free software, also known as "open source."  You can redistribute
 * it and/or modify it under the terms of the GNU General Public License
 * (GPL), version 2, as published by the Free Software Foundation (FSF).
 * 
 * Clack is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GPL for more details.  You should have
 * received a copy of the GPL along with Clack; see the file COPYING.  If
 * not, write to the FSF, 59 Temple Place #330, Boston, MA 02111-1307, USA.
 * 
 * Clack is modelled on and includes some code fragments from MIT's Click
 * system (http://www.pdos.lcs.mit.edu/click/) which is copyright (c)
 * 1999-2000 Massachusetts Institute of Technology and also GPL'ed.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

#include <clack/c/packet.h>
#include <netinet/ip.h>
#include <arpa/inet.h>

static clack_u16_t ip_sequence_number = 0;  // used to tag IP packets

/*
 * IP checksum algorithm.
 */
unsigned short
in_cksum(void *ipv, int len)
{
	unsigned short *ip = ipv;
	unsigned long sum = 0;

	len >>= 1;
	while (len--) {
		sum += *(ip++);
		if (sum > 0xffff)
			sum -= 0xffff;
	}
	return((~sum) & 0x0000ffff);
}


void add_ip_hdr(clack_packet_t *p, 
                unsigned char protocol, unsigned char ttl, 
                struct in_addr src, struct in_addr dst)
{
        struct ip *ip_hdr = clack_packet_UNSHIFT(p, struct ip);

        ip_hdr->ip_v   = IPVERSION;
        ip_hdr->ip_hl  = sizeof(struct ip) >> 2;
        ip_hdr->ip_tos = 0; // ToDo
        ip_hdr->ip_len = htons(clack_packet_length(p));
        ip_hdr->ip_id  = htons(ip_sequence_number++);
        ip_hdr->ip_off = htons(0);  // ToDo: should be arguments to function
        ip_hdr->ip_p   = protocol;
        ip_hdr->ip_ttl = ttl;
        ip_hdr->ip_src = src;
        ip_hdr->ip_dst = dst;
        ip_hdr->ip_sum = 0;

        ip_hdr->ip_sum = in_cksum((unsigned char *) ip_hdr, sizeof(struct ip));
}


