/*
 * Copyright (c) 2000, 2001 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Clack.
 * 
 * Clack is free software, also known as "open source."  You can redistribute
 * it and/or modify it under the terms of the GNU General Public License
 * (GPL), version 2, as published by the Free Software Foundation (FSF).
 * 
 * Clack is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GPL for more details.  You should have
 * received a copy of the GPL along with Clack; see the file COPYING.  If
 * not, write to the FSF, 59 Temple Place #330, Boston, MA 02111-1307, USA.
 * 
 * Clack is modelled on and includes some code fragments from MIT's Click
 * system (http://www.pdos.lcs.mit.edu/click/) which is copyright (c)
 * 1999-2000 Massachusetts Institute of Technology and also GPL'ed.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

#include <oskit/io/netio.h>
#include <oskit/io/bufio.h>
#include <clack/c/packet.h>
#ifdef MEASURE
#include <clack/c/stats.h>
#endif

extern oskit_error_t push(clack_packet_t *p);

oskit_netio_t *nio = 0;

static
oskit_error_t wrapped_push(void *data, oskit_bufio_t *b, oskit_size_t pkt_size);


static
oskit_error_t wrapped_push(void *data, oskit_bufio_t *b, oskit_size_t size)
{
        clack_packet_t *p;

        p = bufio_to_clack_packet(b,size);
#ifdef  MEASURE
#ifdef  PERFMON
	p->counter0 = read_perfcounter(0);
	p->counter1 = read_perfcounter(1);
#endif
	/*
	 * Start clock ticking after the packet is created since
	 * its pointless to compare the packet creation stuff between
	 * usermode, oskit, and linux.
	 */
	p->measure = 1;
	p->created = get_tsc();
#endif
        if (p) {
                return push(p);
        } else {
                return OSKIT_ENOMEM;
        }
}

oskit_error_t
init(void)
{
 	nio = oskit_netio_create(wrapped_push, 0);
        return (!nio);
}


oskit_error_t
fini(void)
{
        if (nio) {
                oskit_netio_release(nio);
                nio = 0;
        }
        return 0;
}
